﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using iikoCardISSApp;
using System.IO;
using System.Windows.Forms;

namespace iikoCardISConsole
{
    public delegate void ObjectDelegate<T>(T obj, params object[] args);

    /// <summary>
    /// Модель приложения
    /// </summary>
    public class Model
    {
        /// <summary>
        /// Конфигурация сервиса изменилась моделью
        /// </summary>
        public event ObjectDelegate<iikoCardISConfig> OnServiceConfigChanged;

        /// <summary>
        ////Конфигурация модели изменилась
        /// </summary>
        public event ObjectDelegate<ModelConfig> OnModelConfigChanged;

        /// <summary>
        /// Запуск модели
        /// </summary>
        public event ObjectDelegate<ModelConfig> OnModelStarted;

        /// <summary>
        /// Конфигурация сервиса, основной объект мониторинга и управления сервисом
        /// </summary>
        public iikoCardISConfig ServiceConfig
        {
            get
            {
                return serviceConfig;
            }
            set
            {
                serviceConfig = value;

                if (OnServiceConfigChanged != null)
                    OnServiceConfigChanged(serviceConfig);
            }
        }
        iikoCardISConfig serviceConfig;

        /// <summary>
        /// Установка новой конфигурации службы
        /// </summary>
        /// <param name="config"></param>
        public void SetNewServiceConfig(iikoCardISConfig config)
        {
            serviceConfig = config;

            config.Save(ModelConfig.ServiceConfigPath);
        }

        /// <summary>
        /// Конфигурация модели
        /// </summary>
        public ModelConfig ModelConfig
        {
            get
            {
                return modelConfig;
            }
            set
            {
                modelConfig = value;

                if (OnModelConfigChanged != null)
                    OnModelConfigChanged(modelConfig);
            }
        }
        ModelConfig modelConfig = new ModelConfig();

        string modelConfigPath = "iikoCardISConsole.cfg";
        string serviceConfigPath = Path.Combine(Environment.CurrentDirectory, "iikoCardIS.cfg");

        public Log Log
        {
            get;
            set;
        }

        /// <summary>
        /// Ядро сервиса для встроенного режима
        /// </summary>
        iikoCardCore Core;

        /// <summary>
        /// Старт модели
        /// </summary>
        /// <param name="configpath"></param>
        public void Start()
        {
            try
            {

                Log = new iikoCardISSApp.Log(Path.Combine(Environment.CurrentDirectory, @"logs\start.log"), Core);



                //загрузка предыдущей конфигурации модели
                modelConfig = ModelConfig.Load(modelConfigPath);

                if (ModelConfig.WorkMode == iikoCardISConsole.ModelConfig.WorkModes.BuildIn)
                {
                    modelConfig.ServiceConfigPath = serviceConfigPath;

                    Core = new iikoCardCore();
                     

                    Core.Start();
                }


                ModelConfig = modelConfig; //срабатывание события

                ConfigWatcher = new FileSystemWatcher(Path.GetDirectoryName(ModelConfig.ServiceConfigPath), Path.GetFileName(ModelConfig.ServiceConfigPath));
                ConfigWatcher.Changed += new FileSystemEventHandler(ConfigWatcher_Changed);
                ConfigWatcher.EnableRaisingEvents = true;
                ServiceConfig = new iikoCardISConfig();
                ConfigWatcher_Changed(null, null);

                if (OnModelStarted != null)
                    OnModelStarted(null, ServiceConfig.MainParameters.Version, ServiceConfig.MainParameters.VersionDate);

                Log.WriteToLog(string.Format(@"Запуск приложения ({0}\{1})", Environment.UserDomainName, Environment.UserName));

            }
            catch 
            {
                throw;
            }
        }

        void ConfigWatcher_Changed(object sender, FileSystemEventArgs e)
        {
            ServiceConfig = ServiceConfig.Load(ModelConfig.ServiceConfigPath);

            if (OnServiceConfigChanged!=null)
                OnServiceConfigChanged(ServiceConfig);
        }

        /// <summary>
        /// Поставить в очередь операцию импорта
        /// </summary>
        /// <param name="file"></param>
        public void EnqueueImportOperation(string file)
        {
            var o = new StandartImportOperation(Core)
            {
                ImportFile = file,
            };

            ServiceConfig.ControlParameters = new iikoCardISConfig.Control()
            {
                PendingOperation = o,
            };

            ServiceConfig.Save(ModelConfig.ServiceConfigPath);

        }

        /// <summary>
        /// Поставить в очередь операцию импорта
        /// </summary>
        /// <param name="file"></param>
        public void EnqueueExportOperation(DateTime from, DateTime to)
        {
            var o = new StandartExportOperation(Core)
            {
                FromDate = from,
                ToDate = to,
                Auto = false
            };

            ServiceConfig.ControlParameters = new iikoCardISConfig.Control()
            {
                PendingOperation = o,
                
            };

            ServiceConfig.Save(ModelConfig.ServiceConfigPath);
        }

        /// <summary>
        /// Остановка модели
        /// </summary>
        public void Stop()
        {
        }

        /// <summary>
        /// Наблюдатель за конфигом сервиса
        /// </summary>
        public FileSystemWatcher ConfigWatcher
        {
            get;
            set;
        }

        /// <summary>
        /// Загрузка нового файла конфигурации
        /// </summary>
        /// <param name="path"></param>
        public void SpecifyConfigFile(string path)
        {
            ServiceConfig = (new iikoCardISConfig()).Load(path);

            ModelConfig.ServiceConfigPath = path;
            SaveModelConfig();
        }

        /// <summary>
        /// Сохранение конфига приложения
        /// </summary>
        public void SaveModelConfig()
        {
            ModelConfig.Save(modelConfigPath);
        }
    }
}
