﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using iikoCardISSApp;
using System.Globalization;

namespace iikoCardISConsole
{
    public partial class Form1 : Form, IMainView
    {
        public Form1()
        {
            InitializeComponent();

        }

        public event ObjectDelegate<string> OnConfigPathSpecified;

        public event ObjectDelegate<iikoCardISConfig> OnConfigChanged;

        public event ObjectDelegate<ModelConfig> OnModelConfigNeedSave;

        public void RaiseException(Exception ex)
        {
            MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        delegate void ServiceConfigDelegate(iikoCardISConfig value);

        public iikoCardISConfig Config
        {
            get
            {
                return new iikoCardISConfig() { MainParameters = (iikoCardISConfig.MainConfig)propertyGrid.SelectedObject };
            }
            set
            {
                if (this.InvokeRequired)
                {
                    this.Invoke(new ServiceConfigDelegate(SetServiceConfig), value);
                }
                else
                {
                    SetServiceConfig(value);
                }
            }
        }

        public void SetCommonProps(params object[] args)
        {
            label13.Text = string.Format("Модуль обмена данными с платежной системой iikoCard через API\r\n{0} 2011 ARBUS\r\nВерсия: {1} от {2}", (char)169, args[0].ToString(), ((DateTime)(args[1])).ToShortDateString());
        }

        void SetServiceConfig(iikoCardISConfig value)
        {
            //основные параметры
            this.propertyGrid.SelectedObject = value.MainParameters;

            //параметры мониторинга
            if (value.MonitoringParameters != null)
            {
                if (value.MonitoringParameters.CurrentOperation != null)
                {
                    toolStripStatusLabel1.Text = "Выполнении операции \""+ value.MonitoringParameters.CurrentOperation.FriendlyName + "\"";
                }
                else
                {
                    toolStripStatusLabel1.Text = "Ожидание";
                }

                if (value.MonitoringParameters.QueueCount > 0)
                    toolStripStatusLabel2.Text = "Операций в очереди: " + value.MonitoringParameters.QueueCount;
                else
                    toolStripStatusLabel2.Text = "";

                //сообщение обратной связи
                if (value.MonitoringParameters.FeedbackMessage!=null)
                    if (value.MonitoringParameters.FeedbackMessage != "")
                    {
                        var s = value.MonitoringParameters.FeedbackMessage;

                        value.MonitoringParameters.FeedbackMessage = null;

                        if (OnConfigChanged != null)
                            OnConfigChanged(Config);

                        //MessageBox.Show(s, "Информация", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        var mf = new MessageForm(s);
                        mf.ShowDialog();
                    }

            }

            //даты операций
            label5.Text = string.Format("{0}", value.MainParameters.LastImportDate);
            label6.Text = string.Format("{0}", value.MainParameters.LastExportDate);
            label9.Text = string.Format("{0}", value.MainParameters.LastSuccefulExportOperDay.ToShortDateString());

            

        }

        private void button5_Click(object sender, EventArgs e)
        {
            if (OnConfigPathSpecified!=null)
                OnConfigPathSpecified(textBox1.Text);
        }

        private void textBox1_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
                button5_Click(null, null);
        }

        bool dontSaveModel = false;

        public void SetModelConfig(ModelConfig c)
        {
            try
            {
                dontSaveModel = true;

                dateTimePicker1.Value = DateTime.Now.Date.AddDays(-1);
                //dateTimePicker2.Value = c.ExportTo;

                if (c.ViewSize != null)
                    if ((c.ViewSize.Width > 100) && (c.ViewSize.Height > 100))
                        this.Size = c.ViewSize;

                if (c.Position != null)
                {
                    this.Location = c.Position;
                }

                groupBox3.Visible = false;

                //режим отображения
                if (c.ViewMode == ModelConfig.ViewModes.User)
                {
                    splitContainer1.Panel2Collapsed = true;
                    splitContainer1.Panel2.Enabled = false;
                }

                if (textBox1.Text != c.ServiceConfigPath)
                {
                    textBox1.Text = c.ServiceConfigPath;
                    button5_Click(null, null);
                }
            }
            catch (Exception)
            {

                throw;
            }
            finally
            {
                dontSaveModel = false;
            }
        }


        private void propertyGrid_PropertyValueChanged(object s, PropertyValueChangedEventArgs e)
        {
            if (OnConfigChanged != null)
                OnConfigChanged(Config);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            /*string outs = "";
            int Dec = 2;
            int Length = 12;
            string Values = (15.65M).ToString("F2");

            string format = "";
            if (Dec != 0)
            {
                format = format.PadRight(Length
                    - Dec - 1, '0');
                format += ".";
                format = format.PadRight(Length, '0');
            }
            else
            {
                format = format.PadRight(Length, '#');
            }
            //Values = Values.Replace('.', ',');
            Values = Values.Replace(".", CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator).Replace(",", CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator);
            string dat = Convert.ToDouble(Values).ToString(format);
            dat = dat.PadLeft(Length);
            //for (int i = 0; i < dat.Length; i++)
            //{
            //    dbfr[i + PositionField] = (byte)dat[i];
            //}
            outs += dat;*/

            //var mf = new MessageForm("И еще было \r\nбы \r\nудобно,\r\n чтобы ширина окна\r\n под сообщения\r\n об экспорте\r\n позволяла опердень \r\nв одну строку видеть без переносов.");
            //var mf = new MessageForm("И еще было бы nудобно,чтобы ширина окнапод сообщенияn об экспорте позволяла опердень в одну строку видеть без переносов.");
            //mf.ShowDialog();
        }

        void RaiseConfigNeedSave()
        {
            if (dontSaveModel) return;

            if (OnModelConfigNeedSave != null)
                OnModelConfigNeedSave(new ModelConfig() { Position = this.Location, ViewSize = this.Size, ExportTo = dateTimePicker2.Value, ExportFrom = dateTimePicker1.Value });
        }

        private void Form1_ResizeEnd(object sender, EventArgs e)
        {
            RaiseConfigNeedSave();
        }

        private void dateTimePicker1_ValueChanged(object sender, EventArgs e)
        {
            /*if (dateTimePicker1.Value > dateTimePicker2.Value)
            {
                MessageBox.Show("Дата должна быть меньше либо равна " + dateTimePicker2.Value.ToShortDateString(), "Предупреждение", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                dateTimePicker1.Value = dateTimePicker2.Value;
                return;
            }*/

            //проверяем что дата "от" не более одного дня в минус от текущей даты
            if (dateTimePicker1.Value > DateTime.Now.Date.AddDays(-1))
            {
                MessageBox.Show("Дата \"С\" не может быть больше "+DateTime.Now.Date.AddDays(-1).ToShortDateString(), "Предупреждение", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                dateTimePicker1.Value = DateTime.Now.Date.AddDays(-1);
                return;
            }

            //меняем дату "до" на +1 день согласно тз
            dateTimePicker2.Value = dateTimePicker1.Value.AddDays(1);

            //RaiseConfigNeedSave();
        }

        private void dateTimePicker2_ValueChanged(object sender, EventArgs e)
        {
            if (dateTimePicker2.Value < dateTimePicker1.Value)
            {
                MessageBox.Show("Дата \"По\" должна быть больше либо равна "+dateTimePicker1.Value.ToShortDateString(), "Предупреждение", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                dateTimePicker2.Value = dateTimePicker1.Value;
                return;
            }

            if (dateTimePicker2.Value > DateTime.Now.Date)
            {
                MessageBox.Show("Дата \"По\" должна быть меньше либо равна текущей дате " + DateTime.Now.Date.ToShortDateString(), "Предупреждение", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                dateTimePicker2.Value = DateTime.Now.Date;
                return;
            }

            //RaiseConfigNeedSave();
        }

        private void Form1_LocationChanged(object sender, EventArgs e)
        {
            RaiseConfigNeedSave();
        }

        /// <summary>
        /// Выбран файл импорта
        /// </summary>
        public event ObjectDelegate<string> OnImportFileSelected;

        /// <summary>
        /// Необходима операция экспорта
        /// </summary>
        public event ObjectDelegate<string> OnExportOperationNeed;


        private void button3_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                if (OnImportFileSelected != null)
                    OnImportFileSelected(openFileDialog1.FileName);
            }
        }

        private void button4_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Синхронизация POS серверов завершена?", "", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
            {
                if (OnExportOperationNeed != null)
                    OnExportOperationNeed("", dateTimePicker1.Value, dateTimePicker2.Value);
            }
        }

    }
}
