﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using iikoCardISSApp;
using System.Windows.Forms;

namespace iikoCardISConsole
{
    /// <summary>
    /// Презентор представлений
    /// </summary>
    public class MainPresentor
    {
        //Модель
        Model model;

        //Основное представление
        IMainView view;

        public MainPresentor(IMainView v)
        {

            model = new Model();

            view = v;

            //подписка на события представления
            view.OnConfigPathSpecified += new ObjectDelegate<string>(view_OnConfigPathSpecified);
            view.OnConfigChanged += new ObjectDelegate<iikoCardISConfig>(view_OnConfigChanged);
            view.OnModelConfigNeedSave += new ObjectDelegate<ModelConfig>(view_OnModelConfigNeedSave);
            view.OnImportFileSelected += new ObjectDelegate<string>(view_OnImportFileSelected);
            view.OnExportOperationNeed += new ObjectDelegate<string>(view_OnExportOperationNeed);

            //подписка на события модели
            model.OnServiceConfigChanged += new ObjectDelegate<iikoCardISConfig>(model_OnConfigChanged);
            model.OnModelConfigChanged += new ObjectDelegate<ModelConfig>(model_OnModelConfigChanged);
            model.OnModelStarted += new ObjectDelegate<ModelConfig>(model_OnModelStarted);

            model.Start();


        }

        void model_OnModelStarted(ModelConfig obj, params object[] args)
        {
            view.SetCommonProps(args);
        }

        void view_OnExportOperationNeed(string obj, params object[] args)
        {
            model.EnqueueExportOperation((DateTime)args[0], (DateTime)args[1]);
        }

        void view_OnImportFileSelected(string obj, params object[] args)
        {
            model.EnqueueImportOperation(obj);
        }

        void view_OnModelConfigNeedSave(ModelConfig obj, params object[] args)
        {
            model.ModelConfig.ExportFrom = obj.ExportFrom;
            model.ModelConfig.ExportTo = obj.ExportTo;
            model.ModelConfig.ViewSize = obj.ViewSize;
            model.ModelConfig.Position = obj.Position;

            model.SaveModelConfig();
        }

        void view_OnConfigChanged(iikoCardISConfig obj, params object[] args)
        {
            model.SetNewServiceConfig(obj);
        }

        /// <summary>
        /// Конфигурация модели поменялась
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="args"></param>
        void model_OnModelConfigChanged(ModelConfig obj, params object[] args)
        {
            try
            {
                view.SetModelConfig(obj);
            }
            catch
            {
               
            }
        }

        /// <summary>
        /// Конфигурация сервиса поменялась
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="args"></param>
        void model_OnConfigChanged(iikoCardISConfig obj, params object[] args)
        {
            try
            {
                view.Config = obj;
            }
            catch (Exception ex)
            {
                view.RaiseException(ex);
            }
        }

        /// <summary>
        /// Указали файл конфигурации
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="args"></param>
        /// <returns></returns>
        void view_OnConfigPathSpecified(string obj, params object[] args)
        {
            try
            {
                //передаем путь модели
                model.SpecifyConfigFile(obj);
            }
            catch (Exception ex)
            {
                view.RaiseException(ex);
            }
        }
    }
}
