﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Collections;
using System.Net.Mail;

namespace iikoCardISSApp
{
    public class Log
    {
        iikoCardCore core;

        /// <summary>
        /// Путь к файлу-логу
        /// </summary>
        public string path;

        /// <summary>
        /// Формат даты в логе
        /// </summary>
        string dateFormat = "dd.MM.yy HH:mm:ss.f";

        /// <summary>
        /// Constructor
        /// </summary>
        public Log(string path, iikoCardCore core)
        {
            this.path = path;
            this.core = core;
        }

        /// <summary>
        /// Отправлять почту
        /// </summary>
        public bool SendMail
        {
            get;
            set;
        }

        public StreamWriter writer = null;

        /// <summary>
        /// Constructor
        /// </summary>
        public Log(string path, string dateFormat)
        {
            this.path = path;
            this.dateFormat = dateFormat;
        }

        /// <summary>
        /// Ежедневный лог
        /// </summary>
        public bool EveryDayLog
        {
            get;
            set;
        }

        /// <summary>
        /// Записать Exception в лог
        /// </summary>
        /// <param name="message">Текст сообщения</param>
        public void WriteToLog(Exception e)
        {
            string message = String.Format("\r\nИсключение: {0} {2}\r\n" +
                                            "Message:\t{1}\r\n",
                                            e.GetType().ToString(),
                                            e.Message == null ? "<null>" : e.Message,
                                            e.StackTrace);

            //кассовые исключения
            /*if (e is PosException || e.GetType().BaseType == typeof(PosException))
            {
                message += String.Format("Tip:\t\t{0}\r\nExSource:\t{1}\r\nCashierTip:\t{2}\r\nInnerEx:\t{3}\r\nMsgType:\t{4}\r\n",
                (e as PosException).Tip == null ? "<null>" : (e as PosException).Tip,
                (e as PosException).ExSource == null ? "<null>" : (e as PosException).ExSource,
                (e as PosException).CashierTip == null ? "<null>" : (e as PosException).CashierTip,
                (e as PosException).InnerException == null ? "<null>" : (e as PosException).InnerException.Message,
                    "");
            }*/

            //переданные в исключении данные
            if (e.Data != null)
                if (e.Data.Count > 0) message += "\r\nДанные исключения:\r\n";
            foreach (DictionaryEntry item in e.Data)
            {
                message += String.Format("Key:\t{0}\r\nValue:\t{1}\r\n\r\n",
                    item.Key.ToString(), item.Value == null ? "<не задан>" : item.Value.ToString());
            }

            WriteToLog(message);
        }

        public void WriteToLog(string message, Log addLog)
        {
            WriteToLog(message);

            addLog.WriteToLog(message);
        }

        /// <summary>
        /// Записать сообщение в лог
        /// </summary>
        /// <param name="message">Текст сообщения</param>
        public void WriteToLog(string message)
        {
            // Текущая дата
            DateTime date = DateTime.Now;

            lock (this)
            {
                {
                    try
                    {
                        var _path = "";

                        var dir = Path.GetDirectoryName(path);

                        if (!Directory.Exists(dir))
                            Directory.CreateDirectory(dir);

                        if (EveryDayLog)
                        {
                            var s = Path.GetFileNameWithoutExtension(path) + string.Format("-{0:yyyy}-{0:MM}-{0:dd}",DateTime.Now);
                            var ext = Path.GetExtension(path);

                            _path = Path.Combine(dir,s + ext);
                        }

                        if (!File.Exists(_path) || writer == null)
                        {
                            //новый поток
                            writer = new StreamWriter(_path, true, Encoding.GetEncoding(1251));
                        }

                        //Stopwatch sw = new Stopwatch();
                        //sw.Start();
                        var line = string.Format("{0} {1}", date.ToString(dateFormat), message);
                        writer.WriteLine(line);
                        writer.Flush();

                        //отправка почты
                        if (core != null && SendMail)
                            if (core.Config.MainParameters.SMTP != null && core.Config.MainParameters.EMail != null)
                            {
                                try
                                {
                                    var smtp = new SmtpClient();
                                    smtp.Host = core.Config.MainParameters.SMTP;
                                    smtp.Port = 25;
                                    using (var mm = new MailMessage())
                                    {
                                        mm.From = new MailAddress("iikoCardIS@arbus.biz");
                                        var adrs = core.Config.MainParameters.EMail.Split(new char[] { ';' }, StringSplitOptions.RemoveEmptyEntries);
                                        foreach (var adr in adrs)
                                        {
                                            mm.To.Add(new MailAddress(adr.Trim()));
                                        }

                                        mm.Subject = "Ошибка iikoCardIS";
                                        mm.Body = line;
                                        smtp.Send(mm);
                                    }
                                }
                                catch
                                {
                                }
                            }
                        
                    }
                    catch
                    {
                        writer = null;
                    }
                }
            }

        }

        /// <summary>
        /// Записать сообщение в лог в виде форматированного сообщения
        /// </summary>
        /// <param name="message"></param>
        /// <param name="args"></param>
        public void WriteToLog(string message, params object[] args)
        {
            WriteToLog(string.Format(message, args));
        }
    }
}
