﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Serialization;
using System.Xml;
using System.IO;
using System.ComponentModel;
using System.Threading;

namespace iikoCardISSApp
{
    /// <summary>
    /// Конфигурация
    /// </summary>
    public class iikoCardISConfig
    {
        public iikoCardISConfig()
        {
            ControlParameters = new Control();
            MainParameters = new MainConfig();
            MonitoringParameters = new Monitoring();
        }

        /// <summary>
        /// Основные конфигурационные свойства
        /// </summary>
        public MainConfig MainParameters
        {
            get;
            set;
        }

        /// <summary>
        /// Внешнее управление службой
        /// </summary>
        public Control ControlParameters
        {
            get;
            set;
        }

        /// <summary>
        /// Мониторинг работы службы
        /// </summary>
        public Monitoring MonitoringParameters
        {
            get;
            set;
        }

        public class MainConfig
        {
            [Description("Версия ядра службы")]
            [Category("Общие параметры")]
            public string Version
            {
                get;
                set;
            }

            [Description("Дата сборки ядра службы")]
            [Category("Общие параметры")]
            public DateTime VersionDate
            {
                get;
                set;
            }

            [Description("Адрес для отправки сервисных сообщений об ошибках")]
            [Category("Общие параметры")]
            public string EMail
            {
                get;
                set;
            }

            [Description("Адрес SMTP сервера")]
            [Category("Общие параметры")]
            public string SMTP
            {
                get;
                set;
            }

            /// <summary>
            /// Путь к файлу конфигурации подключения к службе WCF
            /// </summary>
            [Description("Путь к файлу конфигурации WCF")]
            [Category("Основные параметры")]
            public string WCFConfigFile
            {
                get;
                set;
            }

            /// <summary>
            /// Полный путь к папке с файлом для импорта
            /// </summary>
            [Description("Полный путь к папке с файлом для импорта")]
            [Category("Импорт")]
            public string ImportFilePath
            {
                get;
                set;
            }

            /// <summary>
            /// Время ежедневного импорта
            /// </summary>
            [Description("Время суток, когда сервис автоматически инициирует импорт данных (ЧЧ:ММ:СС)")]
            [Category("Импорт")]
            public string ImportTime
            {
                get;
                set;
            }

            /// <summary>
            /// Полный путь к папке с архивом исходных файлов
            /// </summary>
            [Description("Полный путь к папке с архивом исходных файлов")]
            [Category("Импорт")]
            public string ImportFilePathArchive
            {
                get;
                set;
            }

            [Description("Полный путь к папке файлов экспорта")]
            [Category("Экспорт")]
            public string ExportFilePath
            {
                get;
                set;
            }

            /// <summary>
            /// Полный путь к папке с файлом для автоэкспорта
            /// </summary>
            /*[Description("Полный путь к папке файлов автоматического экспорта")]
            [Category("Экспорт")]
            public string ExportFilePathAuto
            {
                get;
                set;
            }

            /// <summary>
            /// Полный путь к папке с файлом для ручного экспорта
            /// </summary>
            [Description("Полный путь к папке файлов ручного экспорта")]
            [Category("Экспорт")]
            public string ExportFilePathManual
            {
                get;
                set;
            }*/

            /// <summary>
            /// Интервал (мин), через который осуществляется повторный экспорт, в случае если первый не был завершен из-за не полной синхронизации данных кассовых серверов с главным сервером
            /// </summary>
            /*[Description("Интервал (мин), через который осуществляется повторный экспорт, в случае если первый не был завершен из-за не полной синхронизации данных кассовых серверов с главным сервером")]
            [Category("Экспорт")]
            public string SnoozeTime
            {
                get;
                set;
            }*/

            /// <summary>
            /// Время ежедневного экспорта
            /// </summary>
            [Description("Время ежедневного экспорта (ЧЧ:ММ:СС)")]
            [Category("Экспорт")]
            public string ExportTime
            {
                get;
                set;
            }

            /// <summary>
            /// Дата последнего успешного экспорта
            /// </summary>
            [Description("Дата последнего успешного экспорта")]
            [Category("Экспорт")]
            [ReadOnly(true)]
            public DateTime LastSuccefulExportDate
            {
                get;
                set;
            }

            /// <summary>
            /// Дата последнего успешного экспорта
            /// </summary>
            [Description("Дата последнего экспорта")]
            [Category("Экспорт")]
            [ReadOnly(true)]
            public DateTime LastExportDate
            {
                get;
                set;
            }

            /// <summary>
            /// Дата последнего успешного импорта
            /// </summary>
            [Description("Дата последнего импорта")]
            [Category("Экспорт")]
            [ReadOnly(true)]
            public DateTime LastImportDate
            {
                get;
                set;
            }

            /// <summary>
            /// Дата последнего успешно выгруженного опердня
            /// </summary>
            [Description("Дата последнего успешно выгруженного опердня")]
            [Category("Экспорт")]
            [ReadOnly(true)]
            public DateTime LastSuccefulExportOperDay
            {
                get;
                set;
            }

            /// <summary>
            /// Дата последнего успешно выгруженного опердня
            /// </summary>
            [Description("Время начала опердня")]
            [Category("Экспорт")]
            [ReadOnly(true)]
            public string OperDayStart
            {
                get;
                set;
            }

            /// <summary>
            /// Название типа карт
            /// </summary>
            [Description("Название типа карт")]
            [Category("Импорт")]
            public string GroupName
            {
                get;
                set;
            }

            [Description("Название счета")]
            [Category("Импорт")]
            public string AccountName
            {
                get;
                set;
            }

            /// <summary>
            /// Название организации
            /// </summary>
            /*[Description("Название организации")]
            [Category("Импорт")]
            public string Organization
            {
                get;
                set;
            }*/

            /// <summary>
            /// Наименование операции при экспорте
            /// </summary>
            [Description("Задает название операции оплаты для файла экспорта")]
            [Category("Экспорт")]
            public string PaymentNameExport
            {
                get;
                set;
            }

            /// <summary>
            /// Задает название операции оплаты, по которой нужно делать отбор операций в отчете «По операциям»
            /// </summary>
            [Description("Задает название операции оплаты, по которой нужно делать отбор операций в отчете «По операциям»")]
            [Category("Экспорт")]
            public string PaymentNameFilter
            {
                get;
                set;
            }

            /*[Description("Сумма дневной дотации, в копейках")]
            [Category("Экспорт")]
            public int DotationValue
            {
                get;
                set;
            }*/

            [Description("Параметр импорта 1")]
            [Category("Импорт")]
            public string DictionaryParam1
            {
                get;
                set;
            }

            /*
            [Description("Параметр импорта 2")]
            [Category("Импорт")]
            public string DictionaryParam2
            {
                get;
                set;
            }*/
        }

        public class Control
        {
            /// <summary>
            /// Операция ожидающая постановки в очередь
            /// </summary>
            public AbstractOperation PendingOperation
            {
                get;
                set;
            }
        }

        public class Monitoring
        {
            /// <summary>
            /// Сообщение обратной связи
            /// </summary>
            public string FeedbackMessage
            {
                get;
                set;
            }

            /// <summary>
            /// Текущая выполняющаяся операция
            /// </summary>
            public AbstractOperation CurrentOperation
            {
                get;
                set;
            }

            /// <summary>
            /// Количество операций ожидающих в очереди
            /// </summary>
            public int QueueCount
            {
                get;
                set;
            }
        }

        /// <summary>
        /// Флаг блокировки загрузки параметров
        /// Выставляется вручную, сбрасывается автоматом при сохранении
        /// </summary>
        [XmlIgnore]
        public bool LockLoad
        {
            get
            {
                return lockLoad;
            }
            set
            {
                if (value)
                    while (loading) ;
                        //Thread.Sleep(1);

                lockLoad = value;
            }
        }
        bool lockLoad = false;

        /// <summary>
        /// Ожидание доступности файла на запись
        /// </summary>
        /// <param name="path"></param>
        public void WaitForWrite(string path)
        {
            //ждем доступности файла на запись
            while (true)
            {
                FileStream fs = null;
                try
                {
                    fs = File.Open(path, FileMode.Append);
                    break;

                }
                catch (Exception)
                {

                }
                finally
                {
                    if (fs != null)
                        fs.Close();
                }

                Thread.Sleep(5);

            }
        }

        /// <summary>
        /// Объект синхронизации потоков
        /// </summary>
        static public object sync = new object();

        /// <summary>
        /// Сохранить в файл
        /// </summary>
        /// <param name="path"></param>
        public void Save(string path)
        {
            lock (sync)
            {
                /*var cfg = new iikoCardISConfig()
                {
                    MainParameters = new iikoCardISConfig.MainConfig()
                    {
                        DictionaryParam1 = "Баланс",
                        DictionaryParam2 = "300",
                        ExportFilePathAuto = @"c:\out",
                        ImportFilePath = @"c:\in",
                        ImportFilePathArchive = @"c:\in\arc",
                        GroupName = "TestGroup",
                        Organization = "TestOrg"
                    }
                };

                cfg.Save("iikoCardIS.cfg");*/

                XmlSerializer xg = null;

                XmlWriter xw = null;

                try
                {
                    WaitForWrite(path);

                    xg = new XmlSerializer(typeof(iikoCardISConfig));

                    xw = XmlWriter.Create(path, new XmlWriterSettings()
                    {
                        Encoding = Encoding.GetEncoding(1251),
                        Indent = true
                    });

                    xg.Serialize(xw, this);
                }
                catch
                {

                }
                finally
                {
                    if (xw != null)
                        xw.Close();

                    LockLoad = false;
                }
            }
        }

        bool loading = false;

        public iikoCardISConfig Load(string path)
        {
            //блокировка загрузки
            while (LockLoad)
                Thread.Sleep(1);

            loading = true;
            lock (sync)
            {
                iikoCardISConfig cfg = null;

                FileStream fs = null;
                try
                {
                    WaitForWrite(path);

                    fs = File.Open(path, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);

                    XmlSerializer xs = new XmlSerializer(typeof(iikoCardISConfig));

                    cfg = (iikoCardISConfig)xs.Deserialize(fs);
                }
                catch (Exception)
                {
                    throw;
                }
                finally
                {
                    if (fs != null)
                        fs.Close();

                    loading = false;
                }

                return cfg;
            }
        }
    }
}
