﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Resto.Card.Presentation.DTO;
using System.IO;
using DbfDriver;
using Ionic.Zip;
using System.Windows.Forms;
using System.Text.RegularExpressions;

namespace iikoCardISSApp
{
    public class StandartExportOperation : AbstractOperation
    {
        public StandartExportOperation()
        {

        }

        /// <summary>
        /// Режим экспорта
        /// </summary>
        /*public enum ExportMode
        {
            Auto = 0,
            Manual = 1
        }

        /// <summary>
        /// Режим экспорта
        /// </summary>
        public ExportMode Mode
        {
            get;
            set;
        }*/

        public StandartExportOperation(iikoCardCore core) : base(core)
        {

        }

        public override string FriendlyName
        {
            get { return "Экспорт ежедневных операций"; }
        }

        /// <summary>
        /// С какой даты делать выгрузку
        /// </summary>
        public DateTime FromDate
        {
            get;
            set;
        }

        /// <summary>
        /// По какую дату делать выгрузку
        /// </summary>
        public DateTime ToDate
        {
            get;
            set;
        }

        public override void Execute()
        {
            //string exportfile = "";
            //bool dont_delete = false;
            var tables = new List<DbfTable>();
            StringBuilder resultmsg = new StringBuilder(); //результирующее сообщение по экспорту

            //список файлов экспорта
            var exportfiles = new List<string>();
            //список пропускаемых файлов экспорта
            var ignoredexportfiles = new List<string>();

            try
            {
                TimeSpan offset = new TimeSpan();
                TimeSpan.TryParse(Core.Config.MainParameters.OperDayStart, out offset);
                FromDate = new DateTime(FromDate.Year, FromDate.Month, FromDate.Day, 0, 0, 0) + offset;
                ToDate = new DateTime(ToDate.Year, ToDate.Month, ToDate.Day, 0, 0, 0) + offset;                

                //проверка существования директории экспорта
                string exportDir = Core.Config.MainParameters.ExportFilePath;

                if (!Directory.Exists(exportDir))
                    Directory.CreateDirectory(exportDir);

                //using (var fs = new StreamWriter(exportfile+"csv", true, Encoding.GetEncoding(1251)))
                {
                    using (var caller = new WCFCaller(Core.Config.MainParameters.WCFConfigFile))
                    {
                        Core.FullLog.WriteToLog(string.Format("Период выгрузки от {0} до {1}", FromDate.ToShortDateString(), ToDate.ToShortDateString()));

                        //получаем список кассовых серверов и ищем минимальную дату выгрузки
                        //она должна быть больше текущей, это означает что все сервера выгрузили операции и можно начинать наш экспорт

                        //пока проверка не проводится
                        /*var min_server_date = caller.GetRegistredServers().Min(x=>x.WhenLastRequestsExported);

                        Core.FullLog.WriteToLog(string.Format("Минимальная дата выгрузки кассовых серверов {0}", min_server_date));

                        if (min_server_date <= ToDate)
                        {
                            if (Mode == ExportMode.Manual)
                            {
                                lock (Core.Config)
                                {
                                    Core.Config.LockLoad = true;
                                    Core.Config.MonitoringParameters.FeedbackMessage = "Не все данные за запрашиваемый период. Проверьте синхронизацию и измените период";
                                    Core.Config.Save(Core.ConfigFilePath);
                                }
                            }

                            if (Mode == ExportMode.Auto)
                            {
                                //тут надо запустить интервальный монитор с периодом SnoozeTime
                            }

                            throw new Exception("Не все данные за запрашиваемый период");
                        }

                        //сдвиг даты from до даты последнего импорта
                        try
                        {
                            if (FromDate > Core.Config.MainParameters.LastSuccefulExportDate)
                            {
                                FromDate = Core.Config.MainParameters.LastSuccefulExportDate;
                            }
                        }
                        catch { }*/

                        //Core.FullLog.WriteToLog(string.Format("Окончательный период выгрузки от {0} до {1}", FromDate.ToShortDateString(), ToDate.ToShortDateString()));

                        //разбиваем весь диапазон на операционные дни и выгружаем за каждый день отдельно
                        var from = FromDate;
                        var to = ToDate;                        

                        //список карт для ускорения экспорта
                        var cards = new List<CardAccessor>();
                        while (true) //цикл по опердням
                        {
                            //статистика опердня
                            int totalcount = 0; //всего обработано транзакций
                            //decimal dotationsum = 0; //сумма дотаций
                            decimal creditsum = 0M; //оплата в кредит


                            //прибавляем один день к дате от
                            to = from.AddDays(1);
                            to = to.AddSeconds(-1);

                            if (to > ToDate) break; //выход из цикла

                            /*exportfile = Path.Combine(exportDir, string.Format("{1}{0}", from.ToString("yyyyMMdd"), !Auto ? "out" : "out"));

                            if (File.Exists(exportfile + ".csv") || File.Exists(exportfile + ".dbf"))
                            {
                                dont_delete = true;
                                throw new Exception("Файл экспорта " + Path.GetFileName(exportfile) + " уже существует. Экспорт прерван.");
                            }*/

                            //получаем отчет
                            var result = caller.GetReport(from, to, Core.Config.MainParameters.PaymentNameFilter);
                            
                            //словарь код карты - отставшаяся сумма дотации
                            //var dict = new Dictionary<string, decimal>();

                            foreach (var item in result.Items)
                            {
                                //получаем покупателя, чтобы узнать его организацию
                                var cardNumber = ConvertHexToDec(item.CardNumber);
                                string org = "";
                                var c = cards.FirstOrDefault(x => x.Number == cardNumber);
                                if (c == null)
                                {
                                    var cc = caller.GetCard(cardNumber);
                                    org = NormalizeFileName(cc.Owner.OrganizationName);
                                    cards.Add(cc);
                                }
                                else
                                {
                                    org = NormalizeFileName(c.Owner.OrganizationName);
                                }
                                //формируем имя файла
                                string _exportfile = Path.Combine(Core.Config.MainParameters.ExportFilePath, string.Format("{0}_{1}", org, from.ToString("dd-MM-yy")));                                

                                //папка экспорта для дбф по имени организации
                                string _exportfiledbf = Path.Combine(Path.Combine(Path.GetDirectoryName(_exportfile), org) , Path.GetFileName(_exportfile));
                                if (!Directory.Exists(Path.GetDirectoryName(_exportfiledbf)))
                                    Directory.CreateDirectory(Path.GetDirectoryName(_exportfiledbf));

                                if (!exportfiles.Contains(_exportfile))
                                {
                                    if (!ignoredexportfiles.Contains(_exportfile))
                                    {
                                        //проверим что уже не существет файла экспорта
                                        if (File.Exists(_exportfile + ".csv") || File.Exists(_exportfiledbf + ".zip"))
                                        {
                                            //dont_delete = true;
                                            //throw new Exception("Файл экспорта " + Path.GetFileName(exportfile) + " уже существует. Экспорт прерван.");
                                            Core.FullLog.WriteToLog("Файл экспорта " + Path.GetFileName(_exportfile) + " уже существует.", Core.ErrorLog);

                                            ignoredexportfiles.Add(_exportfile); //добавляем в список игнорируемых файлов
                                            continue;
                                        }
                                        else
                                        {
                                            exportfiles.Add(_exportfile); //добавляем в список выгружаемых файлов
                                        }
                                    }
                                    else
                                    {
                                        continue;
                                    }
                                }


                                totalcount++;

                                //проверяем по словарику дотаций
                                /*decimal rest = 0;
                                if (dict.ContainsKey(cardNumber))
                                {
                                    rest = dict[cardNumber]; //покупатель был
                                }
                                else
                                {
                                    //покупателя нет, добавим
                                    //rest = Core.Config.MainParameters.DotationValue / 100.0M;
                                    dict.Add(cardNumber, rest);
                                }

                                //модифицируем остаток ежедневной дотации на сумму операции
                                dict[cardNumber] = dict[cardNumber] + item.TotalValue;*/

                                //если остаток больше или равен нулю то запись пропускается
                                //если меньше, то выгружается
                                //if (dict[cardNumber] < 0)
                                {
                                    //decimal total = dict[cardNumber];
                                    //dict[cardNumber] = 0;

                                    decimal total = item.TotalValue;
                                    creditsum += total;

                                    #region Выгрузка в dbf

                                    //Core.FullLog.WriteToLog("0");

                                    DbfTable table = null;

                                    //if (!File.Exists(_exportfile + ".dbf"))
                                    var t = tables.FirstOrDefault(x => x.FilePath == _exportfiledbf + ".dbf");
                                    if (t == null)
                                    {
                                        //создаем структуру dbf
                                        table = new DbfTable(_exportfiledbf + ".dbf");
                                        table.Header.Type = DbfTable.Headers.Types.dBaseIIIPlusNotMemo;
                                        table.Header.CodePage = DbfTable.Headers.CodePages.Russian_Windows;
                                        int fieldLoc = 1;
                                        table.Header.Fields.Add(new DbfTable.Field() { Name = "DATE", Type = DbfTable.Field.Types.Date, Length = 8, Locating = fieldLoc, Dec = 0x00, Flag = DbfTable.Field.Flags.Nullable, }); fieldLoc += 8;
                                        table.Header.Fields.Add(new DbfTable.Field() { Name = "TIME", Type = DbfTable.Field.Types.Character, Length = 5, Locating = fieldLoc, Dec = 0x00, Flag = DbfTable.Field.Flags.Nullable, }); fieldLoc += 5;
                                        table.Header.Fields.Add(new DbfTable.Field() { Name = "NUMBER", Type = DbfTable.Field.Types.Numeric, Length = 10, Locating = fieldLoc, Dec = 0x00, Flag = DbfTable.Field.Flags.Nullable, }); fieldLoc += 10;
                                        table.Header.Fields.Add(new DbfTable.Field() { Name = "CARD", Type = DbfTable.Field.Types.Numeric, Length = 10, Locating = fieldLoc, Dec = 0x00, Flag = DbfTable.Field.Flags.Nullable, }); fieldLoc += 10;
                                        table.Header.Fields.Add(new DbfTable.Field() { Name = "NAME", Type = DbfTable.Field.Types.Character, Length = 40, Locating = fieldLoc, Dec = 0x00, Flag = DbfTable.Field.Flags.Nullable, }); fieldLoc += 40;
                                        table.Header.Fields.Add(new DbfTable.Field() { Name = "UNIT", Type = DbfTable.Field.Types.Character, Length = 2, Locating = fieldLoc, Dec = 0x00, Flag = DbfTable.Field.Flags.Nullable, }); fieldLoc += 2;
                                        table.Header.Fields.Add(new DbfTable.Field() { Name = "CHECK", Type = DbfTable.Field.Types.Numeric, Length = 10, Locating = fieldLoc, Dec = 0x00, Flag = DbfTable.Field.Flags.Nullable, }); fieldLoc += 10;
                                        table.Header.Fields.Add(new DbfTable.Field() { Name = "SUM", Type = DbfTable.Field.Types.Numeric, Length = 12, Locating = fieldLoc, Dec = 0x02, Flag = DbfTable.Field.Flags.Nullable, }); fieldLoc += 12;
                                        table.Header.Fields.Add(new DbfTable.Field() { Name = "OPER", Type = DbfTable.Field.Types.Character, Length = 20, Locating = fieldLoc, Dec = 0x00, Flag = DbfTable.Field.Flags.Nullable, }); fieldLoc += 20;
                                        table.Header.RecordLength = fieldLoc - 1;

                                        table.Create(DbfTable.Headers.Indexing.CDXIndex, DbfTable.Headers.CodePages.Russian_Windows);

                                        tables.Add(table);
                                    }
                                    else
                                    {
                                        table = t;
                                    }

                                    table.NewRecord();
                                    //Core.FullLog.WriteToLog("1");
                                    table.Header.Fields.Item(0).Value = from.ToString();  //item.Time.ToString() //DATE
                                    //Core.FullLog.WriteToLog("2");
                                    table.Header.Fields.Item(1).Value = item.Time.ToString("HH:mm"); //TIME
                                    //Core.FullLog.WriteToLog("3");
                                    table.Header.Fields.Item(2).Value = GetValue(table.Header.Fields.Item(2), cardNumber); //NUMBER
                                    table.Header.Fields.Item(3).Value = GetValue(table.Header.Fields.Item(3), cardNumber); //CARD
                                    table.Header.Fields.Item(4).Value = ""; //NAME
                                    table.Header.Fields.Item(5).Value = ""; //UNIT
                                    table.Header.Fields.Item(6).Value = GetCheckValue(table.Header.Fields.Item(6), item.Time); //CHECK
                                    //Core.FullLog.WriteToLog((total/* * 100*/).ToString("F2"));
                                    table.Header.Fields.Item(7).Value = (total/* * 100*/).ToString("F2"); //SUM в рублях
                                    table.Header.Fields.Item(8).Value = Core.Config.MainParameters.PaymentNameExport; //OPER
                                    //Core.FullLog.WriteToLog("4");
                                    table.Write();
                                    #endregion

                                    #region Выгрузка в csv
                                    //формируем строку файла
                                    string line = string.Format("{0};{1};{2};{3};{4};{5};{6};{7:F2};{8}\r\n",
                                        //item.Time.ToString("ddMMhhhh"), //Дата транзакции, DATE
                                        //item.Time.ToString("HH:MM"), //Время транзакции, TIME
                                        from.ToShortDateString(), //Дата транзакции, DATE
                                        item.Time.ToString("HH:mm"), //Время транзакции, TIME
                                        GetValue(table.Header.Fields.Item(2), cardNumber), //Внутренний номер счета в БД, NUMBER
                                        GetValue(table.Header.Fields.Item(3), cardNumber), //Номер идентификатора
                                        "",//caller.GetCard(item.CardExternalId).OwnerName, //Имя собственника идентификатора
                                        "", //Номер кассы
                                        GetCheckValue(table.Header.Fields.Item(6), item.Time), //номер чека
                                        total, //сумма операции
                                        Core.Config.MainParameters.PaymentNameExport //вид операции
                                        );

                                    //fs.WriteLine(line);
                                    File.AppendAllText(_exportfile + ".csv", line, Encoding.GetEncoding(1251));
                                    #endregion

                                }
                            }

                            //использованная сумма дотаций считается как: (число сотрудников * сумму дотации) - сумма положительных остатков
                            //dotationsum = (decimal)dict.Count * (decimal)(Core.Config.MainParameters.DotationValue / 100.0M) - dict.Where(x => x.Value > 0).Sum(x => x.Value);

                            //завернем все dbf в архивы
                            /*var dbfs = Directory.GetFiles(Core.Config.MainParameters.ExportFilePath, "*.dbf", SearchOption.TopDirectoryOnly);
                            foreach (var item in dbfs)
                            {
                                using (var zip = new ZipFile())
                                {
                                    zip.AddFile(item);
                                    zip.Save(Path.Combine(Core.Config.MainParameters.ExportFilePath, Path.GetFileNameWithoutExtension(item)+".zip"));
                                }

                                File.Delete(item);
                            }*/

                            //Core.FullLog.WriteToLog(string.Format("Экспортировано {0} операций. Операционный день {1}", result.Items.Length, from.ToShortDateString()));
                            resultmsg.AppendLine(string.Format("Опердень: {0}, обработано транзакций - {1}, на общую сумму - {2:F2} руб", from.ToShortDateString(), totalcount, creditsum));

                            lock (Core.Config)
                            {
                                Core.Config.LockLoad = true;
                                Core.Config.MainParameters.LastSuccefulExportOperDay = from;
                                Core.Config.Save(Core.ConfigFilePath);
                            }

                            from = from.AddDays(1); //сдвиг даты на следующий опердень
                        }

                        Core.FullLog.WriteToLog(resultmsg.ToString());

                        lock (Core.Config)
                        {
                            Core.Config.LockLoad = true;
                            Core.Config.MonitoringParameters.FeedbackMessage = resultmsg.ToString();
                            Core.Config.Save(Core.ConfigFilePath);
                        }
                    }

                    //fs.Close();
                }
            }
            catch
            {
                /*if (File.Exists(exportfile+"csv") && !dont_delete)
                {
                    try
                    {
                        File.Delete(exportfile+"csv");
                    }
                    catch (Exception)
                    {
                    }
                }

                if (File.Exists(exportfile + "dbf") && !dont_delete)
                {
                    try
                    {
                        File.Delete(exportfile+"dbf");
                    }
                    catch (Exception)
                    {
                    }
                }*/

                throw;
            }
            finally
            {
                lock (Core.Config)
                {
                    Core.Config.LockLoad = true;
                    Core.Config.MainParameters.LastExportDate = DateTime.Now;
                    Core.Config.Save(Core.ConfigFilePath);
                }

                /*if (table != null)
                    table.Close();*/

                foreach (var item in tables)
                {
                    item.Close();

                    using (var zip = new ZipFile(
                        Path.Combine(Path.GetDirectoryName(item.FilePath), Path.GetFileNameWithoutExtension(item.FilePath) + ".zip"), Encoding.GetEncoding(866)))
                    {
                        zip.AddFile(item.FilePath, "").FileName = "card.dbf";
                        zip.Save();
                    }

                    File.Delete(item.FilePath);
                }
            }

        }

        /// <summary>
        /// Возвращает значение поля, с учётом его длины.
        /// </summary>
        private string GetValue(DbfTable.Field field, string value)
        {
            return value.Length >= field.Length ? value.Substring(0, field.Length - 1) : value;
        }

        /// <summary>
        /// Возвращает значение поля, с учётом его длины.
        /// </summary>
        private string GetCheckValue(DbfTable.Field field, DateTime value)
        {
            return GetValue(field, value.ToString("HHmm"));
        }

        /// <summary>
        /// Удаляет из файла невалидные символы.
        /// </summary>
        private static string NormalizeFileName(string fileName)
        {
            string regex = String.Format("[{0}]", Regex.Escape(new string(Path.GetInvalidFileNameChars())));
            Regex removeInvalidChars = new Regex(regex, RegexOptions.Singleline | RegexOptions.Compiled | RegexOptions.CultureInvariant);
            return removeInvalidChars.Replace(fileName, "");
        }

        /// <summary>
        /// Конвертиурет 16-й номер карты в десятичный.
        /// </summary>
        private string ConvertHexToDec(string hexNumber)
        {
            int iNumber = int.Parse(hexNumber, System.Globalization.NumberStyles.HexNumber);
            return iNumber.ToString();
        }
    }
}
