﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;
using System.ServiceModel;
using System.Runtime.Serialization;
using Resto.Card.Presentation.DTO;
using System.Diagnostics;

namespace iikoCardISSApp
{
    /// <summary>
    /// Стандартная операция импорта из ежедневного файла карт
    /// </summary>
    public class StandartImportOperation : AbstractOperation
    {
        public enum Limits
        {
            [EnumDescription("Баланс")]
            Limit = 0,
            [EnumDescription("Без лимита оплат")]
            WithoutLimit = 1,
            [EnumDescription("Лимит оплат на один день")]
            LimitOnDay = 2,
            [EnumDescription("Лимит оплат на неделю")]
            LimitOnWeek = 3,
            [EnumDescription("Лимит оплат на месяц")]
            LimitOnMonth = 4
        }

        private class CardImportInfoWrapper
        {
            private CardImportInfo info;
            private Limits limits;

            public CardImportInfoWrapper(CardImportInfo info, Limits limits)
            {
                this.info = info;
                this.limits = limits;
            }

            public CardImportInfo CardImportInfo
            {
                get { return info; }
            }

            public Limits Limits
            {
                get { return limits; }
            }
        }

        public StandartImportOperation()
        {

        }

        public StandartImportOperation(iikoCardCore core): base(core)
        {
            ImportFile = "";
        }

        /// <summary>
        /// Файл импорта
        /// </summary>
        public string ImportFile
        {
            get;
            set;
        }

        public override void Execute()
        {
            var importdate = DateTime.Now;

            //На вход идет файл карт, обрабатываем его посторочно импортируя каждую карту
            if (ImportFile == "")
                throw new Exception("Файл импорта не указан");

            Core.FullLog.WriteToLog("Файл "+ImportFile);

            try
            {
                //счетчики
                int errors = 0, updates = 0, inserts = 0, total = 0;
                //Core.FullLog.WriteToLog("1");
                //получение списка существющих карт
                var cards = new List<Resto.Card.Presentation.DTO.CardAccessor>();
                Core.FullLog.WriteToLog(string.Format("START {0}.", DateTime.Now), Core.FullLog);

                using (var caller = new WCFCaller(Core.Config.MainParameters.WCFConfigFile))
                {
                    List<CardImportInfoWrapper> newImportCards = new List<CardImportInfoWrapper>();
                    List<CardImportInfoWrapper> updatedImportCards = new List<CardImportInfoWrapper>();
                    cards = caller.GetCards();

                    //открываем на чтение
                    using (var sr = new StreamReader(ImportFile, Encoding.GetEncoding(1251)))
                    {
                        while (!sr.EndOfStream)
                        {
                            try
                            {
                                string line = sr.ReadLine();

                                total++;

                                Core.FullLog.WriteToLog(line);

                                var values = line.Split(',');

                                //срок действия карты
                                var expDate = DateTime.ParseExact(values[1], "dd.MM.yyyy", null);
                                expDate = expDate.AddHours(23.999999);

                                //имя владельца
                                var name = values[5].ToString();

                                //номер карты
                                var num = ConvertDecToHex(values[0]);

                                int type;
                                if (!int.TryParse(values[6], out type))
                                {
                                    throw new Exception(string.Format("Неверный тип карты \'{0}\'.", values[6]));
                                }
                                Limits limits = (Limits)type;
                                // для карт без лимита, баланс по умолчанию 100 000.
                                string money = limits == Limits.WithoutLimit ? "100000" : values[7];

                                //импортируем через api iiko
                                var importCard = new CardImportInfo
                                        {
                                            Number = num, //номер карты
                                            ExternalId = num, //трек карты
                                            GroupName = Core.Config.MainParameters.GroupName,
                                            Status = DateTime.Now <= expDate ? ActivityStatus.Active : ActivityStatus.Inactive,
                                            ActivationDate = DateTime.Now,
                                            DeactivationDate = expDate < DateTime.Now ? DateTime.Now : expDate,
                                            Client = new ClientImportInfo
                                                         {
                                                             Code = name,
                                                             Email = "",
                                                             Name = name,
                                                             Organization = new OrganizationImportInfo()
                                                             {
                                                                 Name = values[11],
                                                             }                                                             
                                                         },
                                            CounterValues = new Dictionary<string, string>() { { Core.Config.MainParameters.DictionaryParam1, money } }
                                        };

                                if (cards.Any(x => x.ExternalId == num))
                                {                                                                 
                                    updatedImportCards.Add(new CardImportInfoWrapper(importCard, limits));                                    
                                }
                                else
                                {
                                    newImportCards.Add(new CardImportInfoWrapper(importCard, limits));
                                }

                            }
                            catch (Exception ex)
                            {
                                Core.FullLog.WriteToLog("Ошибка парсинга - " + ex.Message, Core.ErrorLog);
                            }
                        }
                    }

                    //вставка           
                    if (newImportCards.Any())
                    {
                        int step = 100;
                        int pos = 0;

                        while (newImportCards.Count > pos)
                        {
                            if (pos + step > newImportCards.Count)
                            {
                                step = newImportCards.Count - pos - 1;
                            }

                            var list = newImportCards.GetRange(pos, step);
                            caller.ImportCards(list.Select(c => c.CardImportInfo), FilterType.NoFiltering);

                            try
                            {
                                //обновление счетчика "Баланс"
                                foreach (CardImportInfoWrapper info in list)
                                {
                                    Core.FullLog.WriteToLog(string.Format("Вставка карты {0}...", info.CardImportInfo.Number));
                                    var card = caller.GetCard(info.CardImportInfo.Number);
                                    if (UpdateBalance(ref card, info.Limits))
                                    {
                                        caller.UpdateCard(card);
                                    }
                                    inserts++;
                                }
                            }
                            catch (Exception ex)
                            {
                                errors++;
                                Core.FullLog.WriteToLog("Ошибка импорта добавления строки - " + ex.Message, Core.ErrorLog);
                            }
                            pos += step;
                        }
                    }
                    //обновление      
                    if (updatedImportCards.Any())
                    {
                        caller.UpdateCardsByImportInfo(updatedImportCards.Select(c => c.CardImportInfo), AssociationStrategy.ByCardsExternalIds);

                        try
                        {
                            //обновление счетчика "Баланс"
                            foreach (CardImportInfoWrapper info in updatedImportCards)
                            {
                                Core.FullLog.WriteToLog(string.Format("Обновление карты {0}...", info.CardImportInfo.Number));

                                var card = caller.GetCard(info.CardImportInfo.Number);
                                if (UpdateBalance(ref card, info.Limits))
                                {
                                    caller.UpdateCard(card);
                                }
                                updates++;
                            }
                        }
                        catch (Exception ex)
                        {
                            errors++;
                            Core.FullLog.WriteToLog("Ошибка импорта обновления строки - " + ex.Message, Core.ErrorLog);
                        }
                    }

                    Core.FullLog.WriteToLog(string.Format("END {0}.", DateTime.Now), Core.FullLog);

                    string s = string.Format("{0}, Операция {1}. {2}. Обработано карт - {3}, обновлено - {4}, добавлено - {5}, ошибок - {6}",
                        importdate, errors > 0 ? "завершена с ошибками" : "успешно завершена", Path.GetFileName(ImportFile), total, updates, inserts, errors);

                    Core.FullLog.WriteToLog(s);

                    if (!Auto)
                    {
                        lock (Core.Config)
                        {
                            Core.Config.LockLoad = true;
                            Core.Config.MonitoringParameters.FeedbackMessage = s;
                            Core.Config.Save(Core.ConfigFilePath);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Core.FullLog.WriteToLog("Ошибка импорта - " + ex.Message, Core.ErrorLog);
            }
            finally
            {
                //перемещение файла импорта в архив
                try
                {
                    if (!Directory.Exists(Core.Config.MainParameters.ImportFilePathArchive))
                        Directory.CreateDirectory(Core.Config.MainParameters.ImportFilePathArchive);

                    var dt = DateTime.Now;
                    string moveto = string.Format("{0}-{1:D2}-{2:D2}-{3:D2}{4:D2}-{5}", dt.Year, dt.Month, dt.Day, dt.Hour, dt.Minute, Path.GetFileName(ImportFile));

                    File.Move(ImportFile, Path.Combine(Core.Config.MainParameters.ImportFilePathArchive, moveto ));
                }
                catch (Exception ex)
                {
                    Core.FullLog.WriteToLog("Ошибка перемещения файла импорта в архив - "+ex.Message, Core.ErrorLog);
                }

                lock (Core.Config)
                {
                    Core.Config.LockLoad = true;
                    Core.Config.MainParameters.LastImportDate = DateTime.Now;
                    Core.Config.Save(Core.ConfigFilePath);
                }
            }
        }

        public override string FriendlyName
        {
            get { return "Импорт ежедневного файла карт"; }
        }

        private bool UpdateBalance(ref CardAccessor card, Limits limits)
        {
            //обновление счетчика "Баланс"
            if (card != null)
            {
                var acc = card.Accounts.FirstOrDefault(x => x.Name == Core.Config.MainParameters.AccountName);
                if (acc != null)
                {
                    var counter = acc.Counters.FirstOrDefault(x => x.CounterType.Name == Core.Config.MainParameters.DictionaryParam1);

                    if (counter != null)
                    {
                        counter.PeriodControl = new SimplePeriodControl() { IsActive = true, IntervalType = GetIntervalTypeByLimits(limits) };
                        return true;
                    }
                }
            }
            return false;
        }

        /// <summary>
        /// Преобразует тип карты Яндекса в тип IikoCard.
        /// </summary>
        private IntervalType GetIntervalTypeByLimits(Limits limits)
        {
            if (limits == Limits.LimitOnDay || limits == Limits.WithoutLimit)
            {
                return IntervalType.Day;
            }
            if (limits == Limits.LimitOnWeek)
            {
                return IntervalType.Week;
            }
            if (limits == Limits.LimitOnMonth)
            {
                return IntervalType.Month;
            }            
            throw new Exception(string.Format("Для заданного типа карты \'{0}\', не задано преобразование.", limits.GetDescription()));
        }

        private string ConvertDecToHex(string decNumber)
        {
            return String.Format("{0:X}", decNumber);
        }
    }
}
