﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Resto.Card.WCF;
using Resto.Card.Presentation.DTO;
using System.IO;

namespace iikoCardISSApp
{
    public class WCFCaller : IDisposable
    {
        private readonly ServiceCaller caller;

        public WCFCaller(string callerConfig)
        {
            caller = new ServiceCaller();
            caller.ClearCashe();
            caller.SetConfigurationFile(Path.Combine(Directory.GetCurrentDirectory(), callerConfig));
        }

        /// <summary>
        /// Получение списка всех карт
        /// </summary>
        /// <returns></returns>
        public List<CardAccessor> GetCards()
        {
            //var l = new List<Accessor>();
            //l.AddRange(caller.Call<IManagementService, IEnumerable<Accessor>>(service => service.GetAccessors()));

            var l = new List<CardAccessor>();
            l.AddRange(caller.Call<IManagementService, IEnumerable<CardAccessor>>(service => service.GetAllCardAccessorsLight()));

            return l;
        }

        public CardAccessor GetCard(string number)
        {
            try
            {
                Guid addCardId =
                    caller.Call<IManagementService, IEnumerable<CardAccessor>>(service => service.GetAllCardAccessorsLight())
                        .Single(acc => acc.ExternalId == number).Id;
                return
                    caller.Call<IManagementService, CardAccessor>(service => service.GetCardAccessorById(addCardId));
            }
            catch
            {
                return null;
            }
        }

        public CardAccessor GetCard(Guid uid)
        {
            return
                  caller.Call<IManagementService, CardAccessor>(service => service.GetCardAccessorById(uid));
        }

        public void UpdateCardsByImportInfo(IEnumerable<CardImportInfo> cards, AssociationStrategy associationStrategy)
        {
            caller.Call<IManagementService>(s => s.UpdateCardsByImportInfo(cards.ToArray(), associationStrategy));
        }

        public int ImportCards(IEnumerable<CardImportInfo> cards, FilterType filterType)
        {
            return caller.Call<IManagementService, int>(s=>s.ImportCards(cards.ToArray(), filterType));
        }

        public void UpdateCard(CardAccessor card)
        {
            caller.Call<IManagementService>(service => service.UpdateAccessor(card));
        }

        public void AddCard(CardAccessor card)
        {
            caller.Call<IManagementService>(service => service.UpdateAccessor(card));
        }

        public DetailedCardUsageReport GetReport(DateTime from, DateTime to, string serviceName)
        {
            Service s =
                caller.Call<IManagementService, IEnumerable<Service>>(
                    service => service.GetServices()).SingleOrDefault(service => service.Name == serviceName);

            if (s == null)
                throw new Exception(string.Format("Операции \"{0}\" не существует", serviceName));

            var report = new DetailedCardUsageReport()
            {
                From = from,
                To = to,
                IncludeDeletedCards = true,
                ServicesFilter = new[] { s.Id },
            };

            return caller.Call<IReportingService, DetailedCardUsageReport>(service => service.GetDetailedCardUsagesReport(report));
        }

        public IEnumerable<RegisteredServer> GetRegistredServers()
        {
            return caller.Call<IManagementService, IEnumerable<RegisteredServer>>(service => service.GetAllRegisteredServers());
        }

        public void Dispose()
        {
            if (caller != null)
                caller.Dispose();
        }
    }
}
